@@ -200,19 +200,21 @@ from Gnumed.pycommon import gmPG2
 #============================================================
 # short description
 #------------------------------------------------------------
-# use plural form, search-replace get_XXX
-_SQL_get_XXX = u" ""
+# search/replace "" " -> 3 "s
+#
+# search-replace get_XXX, use plural form
+_SQL_get_XXX = u"" "
 	SELECT *, (xmin AS xmin_XXX)
 	FROM XXX.v_XXX
 	WHERE %s
 "" "
 
 class cXxxXxx(gmBusinessDBObject.cBusinessDBObject):
-	" ""Represents ..."" "
+	"" "Represents ..."" "
 
 	_cmd_fetch_payload = _SQL_get_XXX % u"pk_XXX = %s"
 	_cmds_store_payload = [
-		u" ""
+		u"" "
 			-- typically the underlying table name
 			UPDATE xxx.xxx SET
 				-- typically "table_col = %(view_col)s"
@@ -223,10 +225,13 @@ class cXxxXxx(gmBusinessDBObject.cBusinessDBObject):
 					AND
 				xmin = %(xmin_XXX)s
 			RETURNING
-				xmin as xmin_XXX
+				xmin AS xmin_XXX
+				-- also return columns which are calculated in the view used by
+				-- the initial SELECT such that they will further on contain their
+				-- updated value:
 				--, ...
 				--, ...
-		" ""
+		"" "
 	]
 	# view columns that can be updated:
 	_updatable_fields = [
@@ -234,8 +239,8 @@ class cXxxXxx(gmBusinessDBObject.cBusinessDBObject):
 		u'xxx'
 	]
 	#--------------------------------------------------------
-	def format(self):
-		return u'%s' % self
+#	def format(self):
+#		return u'%s' % self
 
 #------------------------------------------------------------
 def get_XXX(order_by=None):
@@ -246,7 +251,7 @@ def get_XXX(order_by=None):
 
 	cmd = _SQL_get_XXX % order_by
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = True)
-	return [ cXxxXxx(row = {'data': r, 'idx': idx, 'pk_field': 'xxx'}) for r in rows ]
+	return [ cXxxXxx(row = {'data': r, 'idx': idx, 'pk_field': 'pk_XXX'}) for r in rows ]
 #------------------------------------------------------------
 def create_xxx(xxx=None, xxx=None):
 
@@ -254,7 +259,7 @@ def create_xxx(xxx=None, xxx=None):
 		u'xxx': xxx,
 		u'xxx': xxx
 	}
-	cmd = u" ""
+	cmd = u"" "
 		INSERT INTO xxx.xxx (
 			xxx,
 			xxx,
@@ -266,15 +271,15 @@ def create_xxx(xxx=None, xxx=None):
 		)
 		RETURNING pk
 		--RETURNING *
-	" ""
+	"" "
 	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True, get_col_idx = False)
 	#rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True, get_col_idx = True)
 
 	return cXxxXxx(aPK_obj = rows[0]['pk'])
-	#return cXxxXxx(row = {'data': r, 'idx': idx, 'pk_field': 'xxx'})
+	#return cXxxXxx(row = {'data': r, 'idx': idx, 'pk_field': 'pk_XXX'})
 #------------------------------------------------------------
-def delete_xxx(xxx=None):
-	args = {'pk': xxx}
+def delete_xxx(pk_XXX=None):
+	args = {'pk': pk_XXX}
 	cmd = u"DELETE FROM xxx.xxx WHERE pk = %(pk)s"
 	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 	return True
@@ -284,7 +289,7 @@ def delete_xxx(xxx=None):
 
 	"""
 	#--------------------------------------------------------
-	def __init__(self, aPK_obj=None, row=None):
+	def __init__(self, aPK_obj=None, row=None, link_obj=None):
 		"""Init business object.
 
 		Call from child classes:
@@ -306,13 +311,13 @@ def delete_xxx(xxx=None):
 		self.__class__._updatable_fields
 
 		if aPK_obj is not None:
-			self.__init_from_pk(aPK_obj=aPK_obj)
+			self.__init_from_pk(aPK_obj = aPK_obj, link_obj = link_obj)
 		else:
 			self._init_from_row_data(row=row)
 
 		self._is_modified = False
 	#--------------------------------------------------------
-	def __init_from_pk(self, aPK_obj=None):
+	def __init_from_pk(self, aPK_obj=None, link_obj=None):
 		"""Creates a new clinical item instance by its PK.
 
 		aPK_obj can be:
@@ -325,7 +330,7 @@ def delete_xxx(xxx=None):
 				the single-value primary key
 		"""
 		self.pk_obj = aPK_obj
-		result = self.refetch_payload()
+		result = self.refetch_payload(link_obj = link_obj)
 		if result is True:
 			self.original_payload = {}
 			for field in self._idx.keys():
@@ -525,7 +530,7 @@ def delete_xxx(xxx=None):
 	def format(self):
 		return u'%s' % self
 	#--------------------------------------------------------
-	def refetch_payload(self, ignore_changes=False):
+	def refetch_payload(self, ignore_changes=False, link_obj=None):
 		"""Fetch field values from backend.
 		"""
 		if self._is_modified:
@@ -542,6 +547,7 @@ def delete_xxx(xxx=None):
 		else:
 			arg = [self.pk_obj]
 		rows, self._idx = gmPG2.run_ro_queries (
+			link_obj = link_obj,
 			queries = [{'cmd': self.__class__._cmd_fetch_payload, 'args': arg}],
 			get_col_idx = True
 		)
