@@ -117,7 +117,9 @@ known_schema_hashes = {
 	17: '161428ee97a00e3bf56168c3a15b7b50',
 	18: 'a0f9efcabdecfb4ddb6d8c0b69c02092',
 	#19: '419e5225259c53dd36ad80d82066ff02'	# 19.0 only
-	19: '9765373098b03fb208332498f34cd4b5'
+	#19: '9765373098b03fb208332498f34cd4b5' # until 19.11
+	19: '57f009a159f55f77525cc0291e0c8b60', # starting with 19.12
+	20: 'baed1901ed4c2f272b56c8cb2c6d88e8'
 }
 
 map_schema_hash2version = {
@@ -138,8 +140,10 @@ map_schema_hash2version = {
 	'0bcf44ca22c479b52976e5eda1de8161': 16,
 	'161428ee97a00e3bf56168c3a15b7b50': 17,
 	'a0f9efcabdecfb4ddb6d8c0b69c02092': 18,
-	#'419e5225259c53dd36ad80d82066ff02': 19	# 19.0. only
-	'9765373098b03fb208332498f34cd4b5': 19
+	#'419e5225259c53dd36ad80d82066ff02': 19	# 19.0 only
+	#'9765373098b03fb208332498f34cd4b5': 19 # until 19.11
+	'57f009a159f55f77525cc0291e0c8b60': 19, # starting with 19.12
+	'baed1901ed4c2f272b56c8cb2c6d88e8': 20
 }
 
 map_client_branch2required_db_version = {
@@ -155,7 +159,8 @@ map_client_branch2required_db_version = {
 	u'1.1': 16,
 	u'1.2': 17,
 	u'1.3': 18,
-	u'1.4': 19
+	u'1.4': 19,
+	u'1.5': 20
 }
 
 # get columns and data types for a given table
@@ -185,33 +190,78 @@ order by
 # only works for single-column FKs but that's fine
 # needs gm-dbo, any-doc won't work
 SQL_foreign_key_name = u"""SELECT
-	tc.*,
-	tc.constraint_schema,
-	tc.constraint_name,
-	tc.table_schema as source_schema,
-	tc.table_name as source_table,
-	kcu.column_name as source_column,
-	ccu.table_schema as target_schema,
-	ccu.table_name as target_table,
-	ccu.column_name as target_column
+	fk_tbl.*,
+	(SELECT nspname FROM pg_catalog.pg_namespace WHERE oid = fk_tbl.connamespace) AS constraint_schema,
+	fk_tbl.conname AS constraint_name,
+	(SELECT nspname FROM pg_catalog.pg_namespace WHERE oid = (SELECT relnamespace FROM pg_class where oid = fk_tbl.conrelid)) AS source_schema,
+	(SELECT relname FROM pg_catalog.pg_class where oid = fk_tbl.conrelid) AS source_table,
+	(SELECT attname FROM pg_catalog.pg_attribute WHERE attnum = fk_tbl.conkey[1] AND attrelid = (%(src_schema)s || '.' || %(src_tbl)s)::regclass) AS source_column,
+	(SELECT nspname FROM pg_catalog.pg_namespace WHERE oid = (SELECT relnamespace FROM pg_class where oid = fk_tbl.confrelid)) AS target_schema,
+	(SELECT relname FROM pg_catalog.pg_class where oid = fk_tbl.confrelid) AS target_table,
+	(SELECT attname FROM pg_catalog.pg_attribute WHERE attnum = fk_tbl.confkey[1] AND attrelid = (%(target_schema)s || '.' || %(target_tbl)s)::regclass) AS target_column
 FROM
-	information_schema.table_constraints tc
-		INNER JOIN information_schema.constraint_column_usage ccu USING (constraint_catalog, constraint_schema, constraint_name)
-			INNER JOIN information_schema.key_column_usage kcu USING (constraint_catalog, constraint_schema, constraint_name)
+	pg_catalog.pg_constraint fk_tbl
 WHERE
-	tc.constraint_type = 'FOREIGN KEY'
+	fk_tbl.contype = 'f'
 		AND
-	kcu.table_schema = %(src_schema)s
+	fk_tbl.conrelid = (%(src_schema)s || '.' || %(src_tbl)s)::regclass
 		AND
-	kcu.table_name = %(src_tbl)s
+	fk_tbl.conkey[1] = (
+		SELECT
+			col_tbl1.attnum
+		FROM
+			pg_catalog.pg_attribute col_tbl1
+		WHERE
+			col_tbl1.attname = %(src_col)s
+				AND
+			col_tbl1.attrelid = (%(src_schema)s || '.' || %(src_tbl)s)::regclass
+	)
 		AND
-	kcu.column_name = %(src_col)s
+	fk_tbl.confrelid = (%(target_schema)s || '.' || %(target_tbl)s)::regclass
 		AND
-	ccu.table_schema = %(target_schema)s
-		AND
-	ccu.table_name = %(target_tbl)s
-		AND
-	ccu.column_name = %(target_col)s"""
+	fk_tbl.confkey[1] = (
+		SELECT
+			col_tbl2.attnum
+		FROM
+			pg_catalog.pg_attribute col_tbl2
+		WHERE
+			col_tbl2.attname = %(target_col)s
+				AND
+			col_tbl2.attrelid = (%(target_schema)s || '.' || %(target_tbl)s)::regclass
+	)
+"""
+
+SQL_get_index_name = u"""
+SELECT
+	(SELECT nspname FROM pg_namespace WHERE pg_namespace.oid = pg_class.relnamespace)
+		AS index_schema,
+	pg_class.relname
+		AS index_name
+FROM
+	pg_class
+WHERE
+	pg_class.oid IN (
+		SELECT
+			indexrelid
+		FROM
+			pg_index
+		WHERE
+			pg_index.indrelid = %(idx_tbl)s::regclass
+				AND
+			pg_index.indnatts = 1		-- only one column in index
+				AND
+			pg_index.indkey[0] IN (
+				SELECT
+					pg_attribute.attnum
+				FROM
+					pg_attribute
+				WHERE
+					pg_attribute.attrelid = %(idx_tbl)s::regclass
+						AND
+					pg_attribute.attname = %(idx_col)s
+				)
+	)
+"""
 
 # =======================================================================
 # module globals API
@@ -369,7 +419,7 @@ def __request_login_params_tui():
 	print "\nPlease enter the required login parameters:"
 	try:
 		login.host = prompted_input(prompt = "host ('' = non-TCP/IP)", default = '')
-		login.database = prompted_input(prompt = "database", default = 'gnumed_v19')
+		login.database = prompted_input(prompt = "database", default = 'gnumed_v20')
 		login.user = prompted_input(prompt = "user name", default = '')
 		tmp = 'password for "%s" (not shown): ' % login.user
 		login.password = getpass.getpass(tmp)
@@ -635,6 +685,21 @@ where
 
 	return rows
 
+#------------------------------------------------------------------------
+def get_index_name(indexed_table=None, indexed_column=None, link_obj=None):
+
+	args = {
+		'idx_tbl': indexed_table,
+		'idx_col': indexed_column
+	}
+	rows, idx = run_ro_queries (
+		link_obj = link_obj,
+		queries = [{'cmd': SQL_get_index_name, 'args': args}],
+		get_col_idx = False
+	)
+
+	return rows
+
 #------------------------------------------------------------------------
 def get_foreign_key_names(src_schema=None, src_table=None, src_column=None, target_schema=None, target_table=None, target_column=None, link_obj=None):
 
@@ -696,6 +761,23 @@ select exists (
 )"""
 	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': (schema, table)}])
 	return rows[0][0]
+
+#------------------------------------------------------------------------
+def function_exists(link_obj=None, schema=None, function=None):
+
+	cmd = u"""
+		SELECT EXISTS (
+			SELECT 1 FROM pg_proc
+			WHERE proname = %(func)s AND pronamespace = (SELECT oid FROM pg_namespace WHERE nspname = %(schema)s)
+		)
+	"""
+	args = {
+		'func': function,
+		'schema': schema
+	}
+	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}])
+	return rows[0][0]
+
 #------------------------------------------------------------------------
 def get_col_indices(cursor = None):
 	if cursor.description is None:
@@ -1128,7 +1210,7 @@ def file2bytea(query=None, filename=None, args=None, conn=None, file_md5=None):
 
 	# insert the data
 	if conn is None:
-		conn = get_raw_connection(readonly=False)
+		conn = get_raw_connection(readonly = False)
 		close_conn = True
 	else:
 		close_conn = False
@@ -1349,7 +1431,7 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 		curs = conn.cursor()
 		curs_close = curs.close
 	else:
-		raise ValueError('link_obj must be cursor, connection or None and not [%s]' % link_obj)
+		raise ValueError('link_obj must be cursor, connection or None but not [%s]' % link_obj)
 
 	for query in queries:
 		if type(query['cmd']) is not types.UnicodeType:
@@ -1574,18 +1656,25 @@ def get_raw_connection(dsn=None, verbose=False, readonly=True):
 	curs = conn.cursor()
 
 	# set access mode
+	conn.set_session(readonly = readonly)
+	conn.set_session(autocommit = readonly)
 	if readonly:
 		_log.debug('access mode [READ ONLY]')
-		cmd = 'set session characteristics as transaction READ ONLY'
-		curs.execute(cmd)
-		cmd = 'set default_transaction_read_only to on'
-		curs.execute(cmd)
+		#conn.set_session(readonly = True)
+		_log.debug('readonly: autocommit=True to avoid <IDLE IN TRANSACTION>')
+#		conn.autocommit = True
+#		cmd = 'set session characteristics as transaction READ ONLY'
+#		curs.execute(cmd)
+#		cmd = 'set default_transaction_read_only to on'
+#		curs.execute(cmd)
 	else:
 		_log.debug('access mode [READ WRITE]')
-		cmd = 'set session characteristics as transaction READ WRITE'
-		curs.execute(cmd)
-		cmd = 'set default_transaction_read_only to off'
-		curs.execute(cmd)
+#		conn.set_session(readonly = False)
+		_log.debug('readwrite: autocommit=False')
+#		cmd = 'set session characteristics as transaction READ WRITE'
+#		curs.execute(cmd)
+#		cmd = 'set default_transaction_read_only to off'
+#		curs.execute(cmd)
 
 	curs.close()
 	conn.commit()
@@ -1957,8 +2046,6 @@ class cAdapterPyDateTime(object):
 #-----------------------------------------------------------------------
 
 # make sure psycopg2 knows how to handle unicode ...
-# intended to become standard
-# test when Squeeze (and thus psycopg2 2.2 becomes Stable
 psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
 psycopg2.extensions.register_type(psycopg2._psycopg.UNICODEARRAY)
 
@@ -1966,6 +2053,9 @@ psycopg2.extensions.register_type(psycopg2._psycopg.UNICODEARRAY)
 # check in 0.9:
 psycopg2.extensions.register_adapter(pydt.datetime, cAdapterPyDateTime)
 
+# turn dict()s into JSON - only works > 9.2
+#psycopg2.extensions.register_adapter(dict, psycopg2.extras.Json)
+
 # do NOT adapt *lists* to "... IN (*) ..." syntax because we want
 # them adapted to "... ARRAY[]..." so we can support PG arrays
 
@@ -2231,10 +2321,20 @@ if __name__ == "__main__":
 	def test_sanity_check_time_skew():
 		sanity_check_time_skew()
 	#--------------------------------------------------------------------
+	def test_get_foreign_key_names():
+		print get_foreign_key_names (
+			src_schema = u'clin',
+			src_table = u'vaccination',
+			src_column = u'fk_episode',
+			target_schema = u'clin',
+			target_table = u'episode',
+			target_column = u'pk'
+		)
+	#--------------------------------------------------------------------
 	def test_get_foreign_key_details():
 		for row in get_foreign_keys2column (
-			schema = u'dem',
-			table = u'identity',
+			schema = u'clin',
+			table = u'episode',
 			column = u'pk'
 		):
 			print '%s.%s references %s.%s.%s' % (
@@ -2348,6 +2448,9 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 			target_table = 'identity',
 			target_column = 'pk'
 		)
+	#--------------------------------------------------------------------
+	def test_get_index_name():
+		print get_index_name(indexed_table = 'clin.vaccination', indexed_column = 'fk_episode')
 
 	#--------------------------------------------------------------------
 	# run tests
@@ -2363,11 +2466,13 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 	#test_is_pg_interval()
 	#test_sanity_check_time_skew()
 	#test_get_foreign_key_details()
+	#test_get_foreign_key_names()
+	test_get_index_name()
 	#test_set_user_language()
 	#test_get_schema_revision_history()
 	#test_run_query()
 	#test_schema_exists()
 	#test_get_foreign_key_names()
-	test_row_locks()
+	#test_row_locks()
 
 # ======================================================================
