@@ -1,4 +1,4 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 __doc__ = """GNUmed general tools."""
 
 #===========================================================================
@@ -10,6 +10,7 @@ import re as regex, sys, os, os.path, csv, tempfile, logging, hashlib
 import platform
 import subprocess
 import decimal
+import getpass
 import cPickle, zlib
 import xml.sax.saxutils as xml_tools
 
@@ -49,15 +50,21 @@ u_one_quarter = u'\u00BC'
 u_one_half = u'\u00BD'
 u_three_quarters = u'\u00BE'
 u_multiply = u'\u00D7'						# x
+u_greek_ALPHA = u'\u0391'
+u_greek_alpha = u'\u03b1'
+u_greek_OMEGA = u'\u03A9'
+u_greek_omega = u'\u03c9'
 u_triangular_bullet = u'\u2023'				# triangular bullet  (>)
 u_ellipsis = u'\u2026'						# ...
 u_euro = u'\u20AC'							# EURO sign
 u_numero = u'\u2116'						# No. / # sign
 u_down_left_arrow = u'\u21B5'				# <-'
 u_left_arrow = u'\u2190'					# <--
+u_up_arrow = u'\u2191'
 u_right_arrow = u'\u2192'					# -->
+u_down_arrow = u'\u2193'
 u_left_arrow_with_tail = u'\u21a2'			# <--<
-u_sum = u'\u2211'
+u_sum = u'\u2211'							# sigma
 u_almost_equal_to = u'\u2248'				# approximately / nearly / roughly
 u_corresponds_to = u'\u2258'
 u_infinity = u'\u221E'
@@ -101,14 +108,23 @@ def handle_uncaught_exception_console(t, v, tb):
 #===========================================================================
 # path level operations
 #---------------------------------------------------------------------------
-def mkdir(directory=None):
+def mkdir(directory=None, mode=None):
 	try:
-		os.makedirs(directory)
+		if mode is None:
+			os.makedirs(directory)
+		else:
+			old_umask = os.umask(0)
+			os.makedirs(directory, mode)
+			os.umask(old_umask)
 	except OSError, e:
 		if (e.errno == 17) and not os.path.isdir(directory):
 			raise
 	return True
 
+#---------------------------------------------------------------------------
+def dir_is_empty(directory=None):
+	return len(os.listdir(directory)) == 0
+
 #---------------------------------------------------------------------------
 class gmPaths(gmBorg.cBorg):
 	"""This class provides the following paths:
@@ -119,7 +135,7 @@ class gmPaths(gmBorg.cBorg):
 	.user_config_dir
 	.system_config_dir
 	.system_app_data_dir
-	.tmp_dir (readonly)
+	.tmp_dir
 	"""
 	def __init__(self, app_name=None, wx=None):
 		"""Setup pathes.
@@ -191,13 +207,20 @@ class gmPaths(gmBorg.cBorg):
 		# temporary directory
 		try:
 			self.__tmp_dir_already_set
-			_log.debug('temp dir already set')
+			_log.debug(u'temp dir already set')
 		except AttributeError:
+			_log.info(u'initial temp dir: %s', tempfile.gettempdir())
+			# /tmp/gnumed/
 			tmp_base = os.path.join(tempfile.gettempdir(), app_name)
-			mkdir(tmp_base)
-			_log.info('previous temp dir: %s', tempfile.gettempdir())
+			mkdir(tmp_base, 0o777)
 			tempfile.tempdir = tmp_base
-			_log.info('intermediate temp dir: %s', tempfile.gettempdir())
+			_log.info(u'level 1 intermediate temp dir: %s', tempfile.gettempdir())
+			# /tmp/gnumed/$USER/
+			tmp_base = os.path.join(tempfile.gettempdir(), getpass.getuser())
+			mkdir(tmp_base, 0o700)
+			tempfile.tempdir = tmp_base
+			_log.info(u'level 2 intermediate temp dir: %s', tempfile.gettempdir())
+			# /tmp/gnumed/$USER/gm-*/
 			self.tmp_dir = tempfile.mkdtemp(prefix = r'gm-')
 
 		self.__log_paths()
@@ -479,6 +502,8 @@ def get_unique_filename(prefix=None, suffix=None, tmp_dir=None):
 def import_module_from_directory(module_path=None, module_name=None, always_remove_path=False):
 	"""Import a module from any location."""
 
+	_log.debug('CWD: %s', os.getcwd())
+
 	remove_path = always_remove_path or False
 	if module_path not in sys.path:
 		_log.info('appending to sys.path: [%s]' % module_path)
@@ -686,16 +711,16 @@ def input2int(initial=None, minval=None, maxval=None):
 		int_val = int(val)
 	except (TypeError, ValueError):
 		_log.exception('int(%s) failed', val)
-		return False, val
+		return False, initial
 
 	if minval is not None:
 		if int_val < minval:
 			_log.debug('%s < min (%s)', val, minval)
-			return False, val
+			return False, initial
 	if maxval is not None:
 		if int_val > maxval:
 			_log.debug('%s > max (%s)', val, maxval)
-			return False, val
+			return False, initial
 
 	return True, int_val
 #---------------------------------------------------------------------------
@@ -727,6 +752,14 @@ def strip_trailing_empty_lines(lines=None, text=None, eol=u'\n', return_list=Tru
 
 	return eol.join(lines)
 #---------------------------------------------------------------------------
+def strip_empty_lines(lines=None, text=None, eol=u'\n', return_list=True):
+	return strip_trailing_empty_lines (
+		lines = strip_leading_empty_lines(lines = lines, text = text, eol = eol, return_list = True),
+		text = None,
+		eol = eol,
+		return_list = return_list
+	)
+#---------------------------------------------------------------------------
 def wrap(text=None, width=None, initial_indent=u'', subsequent_indent=u'', eol=u'\n'):
 	"""A word-wrap function that preserves existing line breaks
 	and most spaces in the text. Expects that existing line
@@ -780,7 +813,7 @@ def xml_escape_string(text=None):
 	"""check for special XML characters and transform them"""
 	return xml_tools.escape(text)
 #---------------------------------------------------------------------------
-def tex_escape_string(text=None, replace_known_unicode=True):
+def tex_escape_string(text=None, replace_known_unicode=True, replace_eol=False, keep_visual_eol=False):
 	"""check for special TeX characters and transform them"""
 
 	text = text.replace(u'\\', u'\\textbackslash')
@@ -794,6 +827,11 @@ def tex_escape_string(text=None, replace_known_unicode=True):
 	text = text.replace(u'#', u'\\#')
 	text = text.replace(u'$', u'\\$')
 	text = text.replace(u'_', u'\\_')
+	if replace_eol:
+		if keep_visual_eol:
+			text = text.replace(u'\n', u'\\newline \n')
+		else:
+			text = text.replace(u'\n', u'\\newline ')
 
 	if replace_known_unicode:
 		# this should NOT be replaced for Xe(La)Tex
@@ -806,6 +844,18 @@ def xetex_escape_string(text=None):
 	# as opposed to LaTeX, except true unicode chars
 	return tex_escape_string(text = text, replace_known_unicode = False)
 #---------------------------------------------------------------------------
+__html_escape_table = {
+	u"&": u"&amp;",
+	u'"': u"&quot;",
+	u"'": u"&apos;",
+	u">": u"&gt;",
+	u"<": u"&lt;",
+}
+
+def html_escape_string(text=None):
+	return "".join(__html_escape_table.get(char, char) for char in text)
+
+#---------------------------------------------------------------------------
 def prompted_input(prompt=None, default=None):
 	"""Obtains entry from standard input.
 
@@ -1161,7 +1211,7 @@ second line\n
 		print xml_escape_string(u'&')
 	#-----------------------------------------------------------------------
 	def test_tex_escape():
-		tests = [u'\\', u'^', u'~', u'{', u'}', u'%',  u'&', u'#', u'$', u'_', u_euro]
+		tests = [u'\\', u'^', u'~', u'{', u'}', u'%',  u'&', u'#', u'$', u'_', u_euro, u'abc\ndef\n\n1234']
 		tests.append(u'  '.join(tests))
 		for test in tests:
 			print u'%s:' % test, tex_escape_string(test)
@@ -1206,6 +1256,9 @@ second line\n
 		for t in tests:
 			print "[%s] -> [%s]" % (t, fname_stem(t))
 	#-----------------------------------------------------------------------
+	def test_dir_is_empty():
+		print sys.argv[2], 'empty:', dir_is_empty(sys.argv[2])
+	#-----------------------------------------------------------------------
 	#test_coalesce()
 	#test_capitalize()
 	#test_import_module()
@@ -1225,7 +1278,8 @@ second line\n
 	#test_xml_escape()
 	#test_gpg_decrypt()
 	#test_strip_trailing_empty_lines()
-	test_fname_stem()
-	#test_tex_escape()
+	#test_fname_stem()
+	test_tex_escape()
+	#test_dir_is_empty()
 
 #===========================================================================
