@@ -27,6 +27,7 @@ from Gnumed.business import gmFamilyHistory
 from Gnumed.business import gmVaccination
 from Gnumed.business import gmDocuments
 from Gnumed.business import gmProviderInbox
+from Gnumed.business import gmExternalCare
 
 from Gnumed.wxpython import gmRegetMixin
 from Gnumed.wxpython import gmDemographicsWidgets
@@ -39,6 +40,7 @@ from Gnumed.wxpython import gmFamilyHistoryWidgets
 from Gnumed.wxpython import gmVaccWidgets
 from Gnumed.wxpython import gmDocumentWidgets
 from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmPregWidgets
 
 
 _log = logging.getLogger('gm.patient')
@@ -118,7 +120,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	# threads, dispatcher signals etc.
 	def __register_interests(self):
 		# client internal signals
-		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
+		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 
 		# database change signals
@@ -140,15 +142,18 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		gmDispatcher.connect(signal = u'clin.family_history_mod_db', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'clin.procedure_mod_db', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'clin.vaccination_mod_db', receiver = self._on_post_patient_selection)
-		#gmDispatcher.connect(signal = u'gm_table_mod', receiver = self._on_post_patient_selection)
+		#gmDispatcher.connect(signal = u'clin.external_care_mod_db', receiver = self._on_post_patient_selection)
 
 		gmDispatcher.connect(signal = u'dem.message_inbox_mod_db', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'clin.test_result_mod_db', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'clin.reviewed_test_results_mod_db', receiver = self._on_post_patient_selection)
 		gmDispatcher.connect(signal = u'blobs.doc_med_mod_db', receiver = self._on_post_patient_selection)
 
+		# generic signal
+		gmDispatcher.connect(signal = u'gm_table_mod', receiver = self._on_post_patient_selection)
+
 		# synchronous signals
-#		self.__pat.register_pre_selection_callback(callback = self._pre_selection_callback)
+#		self.__pat.register_before_switching_from_patient_callback(callback = self._before_switching_from_patient_callback)
 #		gmDispatcher.send(signal = u'register_pre_exit_callback', callback = self._pre_exit_callback)
 
 		self._PRW_encounter_range.add_callback_on_selection(callback = self._on_encounter_range_selected)
@@ -156,17 +161,17 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	def _on_encounter_range_selected(self, data):
 		wx.CallAfter(self.__refresh_encounters, patient = gmPerson.gmCurrentPatient())
 	#--------------------------------------------------------
-	def _on_pre_patient_selection(self):
+	def _on_pre_patient_unselection(self):
 		# only empty out here, do NOT access the patient
 		# or else we will access the old patient while it
 		# may not be valid anymore ...
-		wx.CallAfter(self.__reset_ui_content)
+		self.__reset_ui_content()
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#--------------------------------------------------------
 	def _on_episode_issue_mod_db(self):
-		wx.CallAfter(self._schedule_data_reget)
+		self._schedule_data_reget()
 	#-----------------------------------------------------
 	# reget-on-paint mixin API
 	#-----------------------------------------------------
@@ -204,23 +209,19 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			return
 
 		now = gmDateTime.pydt_now_here()
-		list_items.append(_('Latest: %s ago (%s %s %s %s%s)') % (
+		list_items.append(_('Latest: %s ago (%s %s%s%s%s)') % (
 			gmDateTime.format_interval_medically(now - most_recent['clin_when']),
 			most_recent['unified_abbrev'],
 			most_recent['unified_val'],
-			most_recent['val_unit'],
+			gmTools.coalesce(most_recent['val_unit'], u'', u' %s'),
 			gmTools.coalesce(most_recent['abnormality_indicator'], u'', u' %s'),
-			gmTools.bool2subst(most_recent['reviewed'], u'', u' %s' % gmTools.u_writing_hand)
+			gmTools.bool2subst(most_recent['reviewed'], u'', (u' %s' % gmTools.u_writing_hand))
 		))
 		list_data.append(most_recent)
 		most_recent_needs_red = False
-		#if most_recent['is_technically_abnormal'] is True:
 		if most_recent.is_considered_abnormal is True:
 			if most_recent['is_clinically_relevant']:
 				most_recent_needs_red = True
-#		else:
-#			if most_recent['abnormality_indicator'] not in [None, u'']:
-#				most_recent_needs_red = True
 
 		unsigned = emr.get_unsigned_results(order_by = u"(trim(coalesce(abnormality_indicator), '') <> '') DESC NULLS LAST, unified_abbrev")
 		no_of_reds = 0
@@ -230,10 +231,10 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			if result['abnormality_indicator'] is not None:
 				if result['abnormality_indicator'].strip() != u'':
 					no_of_reds += 1
-			list_items.append(_('%s %s %s %s (%s ago, %s)') % (
+			list_items.append(_('%s %s%s%s (%s ago, %s)') % (
 				result['unified_abbrev'],
 				result['unified_val'],
-				result['val_unit'],
+				gmTools.coalesce(result['val_unit'], u'', u' %s'),
 				gmTools.coalesce(result['abnormality_indicator'], u'', u' %s'),
 				gmDateTime.format_interval_medically(gmDateTime.pydt_now_here() - result['clin_when']),
 				gmTools.u_writing_hand
@@ -260,7 +261,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 #			if wx.GetKeyState(wx.WXK_CONTROL):
 #				if isinstance(data, gmProviderInbox.cInboxMessage):
 #					xxxxxxxxx
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmMeasurementsGridPlugin')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmMeasurementsGridPlugin')
 		return
 	#-----------------------------------------------------
 	#-----------------------------------------------------
@@ -327,41 +328,59 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 
 		data = self._LCTRL_inbox.get_selected_item_data(only_one = True)
 
+		# if it is a dynamic hint open the URL for that
 		if isinstance(data, gmProviderInbox.cDynamicHint):
 			if data['url'] is not None:
 				gmNetworkTools.open_url_in_browser(data['url'])
 			return
 
+		# holding down <CTRL> when double-clicking an inbox
+		# item indicates the desire to delete it
 		# <ctrl> down ?
-		if not wx.GetKeyState(wx.WXK_CONTROL):
-			wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmProviderInboxPlugin')
+		if wx.GetKeyState(wx.WXK_CONTROL):
+			# better safe than sorry: can only delete real inbox items
+			if data is None:
+				return
+			if not isinstance(data, gmProviderInbox.cInboxMessage):
+				return
+			delete_it = gmGuiHelpers.gm_show_question (
+				question = _('Do you really want to\ndelete this inbox message ?'),
+				title = _('Deleting inbox message')
+			)
+			if not delete_it:
+				return
+			gmProviderInbox.delete_inbox_message(inbox_message = data['pk_inbox_message'])
 			return
 
 		if data is None:
-			wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmProviderInboxPlugin')
+			gmDispatcher.send(signal = 'display_widget', name = 'gmProviderInboxPlugin')
 			return
 
 		if not isinstance(data, gmProviderInbox.cInboxMessage):
-			wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmProviderInboxPlugin')
+			gmDispatcher.send(signal = 'display_widget', name = 'gmProviderInboxPlugin')
 			return
 
-		delete_it = gmGuiHelpers.gm_show_question (
-			question = _('Do you really want to\ndelete this inbox message ?'),
-			title = _('Deleting inbox message')
-		)
-		if not delete_it:
-			return
-
-		gmProviderInbox.delete_inbox_message(inbox_message = data['pk_inbox_message'])
+		gmDispatcher.send(signal = 'display_widget', name = 'gmProviderInboxPlugin', filter_by_active_patient = True)
 		return
 	#-----------------------------------------------------
 	#-----------------------------------------------------
 	def __refresh_documents(self, patient=None):
-		doc_folder = patient.get_document_folder()
 
 		list_items = []
 		list_data = []
 
+		# export area items
+		item_count = len(patient.export_area.items)
+		if item_count == 1:
+			list_items.append(_(u'Export area: 1 item'))
+			list_data.append(u'')
+		if item_count > 1:
+			list_items.append(_(u'Export area: %s items') % item_count)
+			list_data.append(u'')
+
+		doc_folder = patient.get_document_folder()
+
+		# unsigned docs first
 		docs = doc_folder.get_unsigned_documents()
 		no_of_unsigned = len(docs)
 		for doc in docs:
@@ -372,6 +391,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			))
 			list_data.append(doc)
 
+		# other, signed docs second
 		docs = doc_folder.get_documents(order_by = u'ORDER BY clin_when DESC', exclude_unsigned = True)
 		for doc in docs[:5]:
 			list_items.append(u'%s %s' % (
@@ -391,7 +411,11 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		self._LCTRL_documents.set_data(data = list_data)
 
 		if no_of_unsigned > 0:
-			for idx in range(no_of_unsigned):
+			start_idx = 0
+			if item_count > 0:
+				start_idx = 1
+			end_idx = no_of_unsigned + start_idx
+			for idx in range(start_idx, end_idx):
 				self._LCTRL_documents.SetItemTextColour(idx, wx.NamedColour('RED'))
 	#-----------------------------------------------------
 	def _calc_documents_list_item_tooltip(self, data):
@@ -415,7 +439,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 						gmDocumentWidgets.review_document(parent = self, document = data)
 					return
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmShowMedDocs')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmShowMedDocs')
 		return
 	#-----------------------------------------------------
 	#-----------------------------------------------------
@@ -536,7 +560,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		if type(data) == type({}):
 			key, val = data.items()[0]
 			if key == 'wlist':
-				wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmWaitingListPlugin')
+				gmDispatcher.send(signal = 'display_widget', name = 'gmWaitingListPlugin')
 				return
 			if key == 'stay':
 				wx.CallAfter(gmEMRStructWidgets.manage_hospital_stays, parent = self)
@@ -549,27 +573,31 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		emr = patient.get_emr()
 
 		sort_key_list = []
+		date_format4sorting = '%Y %m %d %H %M %S'
 		data = {}
 
-		issues = [
-			i for i in emr.get_health_issues()
-			if ((i['clinically_relevant'] is False) or (i['is_active'] is False))
-		]
-		for issue in issues:
-			last_encounter = emr.get_last_encounter(issue_id = issue['pk_health_issue'])
-			if last_encounter is None:
-				last = issue['modified_when'].strftime('%m/%Y')
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(issue['modified_when'], format = '%Y %m %d %H %M %S'), issue['pk_health_issue'])
+		# undated entries
+		# pregnancy
+		edc = emr.EDC
+		if edc is not None:
+			sort_key = u'99999 edc'
+			if emr.EDC_is_fishy:
+				label = _(u'EDC (!?!): %s') % gmDateTime.pydt_strftime(edc, format = '%Y %b %d')
+				tt = _(
+					u'The Expected Date of Confinement is rather questionable.\n'
+					u'\n'
+					u'Please check patient age, patient gender, time until/since EDC.'
+				)
 			else:
-				last = last_encounter['last_affirmed'].strftime('%m/%Y')
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(last_encounter['last_affirmed'], format = '%Y %m %d %H %M %S'), issue['pk_health_issue'])
+				label = _(u'EDC: %s') % gmDateTime.pydt_strftime(edc, format = '%Y %b %d')
+				tt = u''
 			sort_key_list.append(sort_key)
-			data[sort_key] = [u'%s %s' % (last, issue['description']), issue]
-		del issues
+			data[sort_key] = [label, tt]
 
+		# family history
 		fhxs = emr.get_family_history()
 		for fhx in fhxs:
-			sort_key = u'0000 %s::%s' % (fhx['l10n_relation'], fhx['pk_family_history'])
+			sort_key = u'99998 %s::%s' % (fhx['l10n_relation'], fhx['pk_family_history'])
 			#gmDateTime.pydt_strftime(fhx['when_known_to_patient'], format = '%Y %m %d %H %M %S')
 			label = u'%s: %s%s' % (
 				fhx['l10n_relation'],
@@ -580,17 +608,28 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			data[sort_key] = [label, fhx]
 		del fhxs
 
+		# dated entries
+		issues = [
+			i for i in emr.get_health_issues()
+			if ((i['clinically_relevant'] is False) or (i['is_active'] is False))
+		]
+		for issue in issues:
+			last_encounter = emr.get_last_encounter(issue_id = issue['pk_health_issue'])
+			if last_encounter is None:
+				last = gmDateTime.pydt_strftime(issue['modified_when'], format = '%Y %b')
+				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(issue['modified_when'], format = date_format4sorting), issue['pk_health_issue'])
+			else:
+				last = gmDateTime.pydt_strftime(last_encounter['last_affirmed'], format = '%Y %b')
+				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(last_encounter['last_affirmed'], format = date_format4sorting), issue['pk_health_issue'])
+			sort_key_list.append(sort_key)
+			data[sort_key] = [u'%s %s' % (last, issue['description']), issue]
+		del issues
+
 		stays = emr.get_hospital_stays()
 		for stay in stays:
-			if stay['discharge'] is None:
-				discharge = gmTools.u_ellipsis
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(stay['admission'], format = '%Y %m %d %H %M %S'), stay['pk_hospital_stay'])
-			else:
-				discharge = u''
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(stay['discharge'], format = '%Y %m %d %H %M %S'), stay['pk_hospital_stay'])
-			label = u'%s%s %s: %s' % (
-				gmDateTime.pydt_strftime(stay['admission'], format = '%Y %b %d'),
-				discharge,
+			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(stay['admission'], format = date_format4sorting), stay['pk_hospital_stay'])
+			label = u'%s %s: %s' % (
+				gmDateTime.pydt_strftime(stay['admission'], format = '%Y %b'),
 				stay['hospital'],
 				stay['episode']
 			)
@@ -600,12 +639,9 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 
 		procs = emr.get_performed_procedures()
 		for proc in procs:
-			if proc['is_ongoing']:
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(proc['clin_when'], format = '%Y %m %d %H %M %S'), proc['pk_procedure'])
-			else:
-				sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(proc['clin_end'], format = '%Y %m %d %H %M %S'), proc['pk_procedure'])
+			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(proc['clin_when'], format = date_format4sorting), proc['pk_procedure'])
 			label = u'%s%s %s' % (
-				gmDateTime.pydt_strftime(proc['clin_when'], format = '%Y %b %d'),
+				gmDateTime.pydt_strftime(proc['clin_when'], format = '%Y %b'),
 				gmTools.bool2subst(proc['is_ongoing'], gmTools.u_ellipsis, u'', u''),
 				proc['performed_procedure']
 			)
@@ -616,7 +652,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		vaccs = emr.get_latest_vaccinations()
 		for ind, tmp in vaccs.items():
 			tmp, vacc = tmp
-			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(vacc['date_given'], format = '%Y %m %d %H %M %S'), vacc['pk_vaccination'])
+			sort_key = u'%s::%s' % (gmDateTime.pydt_strftime(vacc['date_given'], format = date_format4sorting), vacc['pk_vaccination'])
 			label = _('%s Vacc: %s') % (
 				gmDateTime.pydt_strftime(vacc['date_given'], format = '%Y %b'),
 				ind
@@ -668,6 +704,12 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				date_format = '%Y %b %d'
 			))
 
+		# EDC
+		if isinstance(data, basestring):
+			if data == u'':
+				return None
+			return data
+
 		return None
 	#-----------------------------------------------------
 	def _on_history_item_activated(self, event):
@@ -675,6 +717,10 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		if data is None:
 			return
 
+		if isinstance(data, basestring):
+			gmPregWidgets.calculate_edc(parent = self, patient = gmPerson.gmCurrentPatient())
+			return
+
 		# <ctrl> down ?
 		if wx.GetKeyState(wx.WXK_CONTROL):
 			if isinstance(data, gmEMRStructItems.cHealthIssue):
@@ -695,7 +741,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			return
 
 		if isinstance(data, gmEMRStructItems.cHealthIssue):
-			wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmEMRBrowserPlugin')
+			gmDispatcher.send(signal = 'display_widget', name = 'gmEMRBrowserPlugin')
 			return
 		if isinstance(data, gmFamilyHistory.cFamilyHistory):
 			FamilyHistoryWidgets.manage_family_history(parent = self)
@@ -773,7 +819,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				wx.CallAfter(gmMedicationWidgets.edit_intake_of_substance, parent = self, substance = data)
 				return
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmCurrentSubstancesPlugin')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmCurrentSubstancesPlugin')
 	#-----------------------------------------------------
 	#-----------------------------------------------------
 	def __refresh_contacts(self, patient=None):
@@ -817,6 +863,15 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 			list_items.append(_('in-praxis: %s') % provider.identity['description_gender'])
 			list_data.append(provider)
 
+		care = emr.get_external_care_items()
+		for item in care:
+			list_items.append(_('care: %s%s@%s') % (
+				gmTools.coalesce(item['provider'], u'', u'%s, '),
+				item['unit'],
+				item['organization']
+			))
+			list_data.append(item)
+
 		self._LCTRL_contacts.set_string_items(items = list_items)
 		self._LCTRL_contacts.set_data(data = list_data)
 		if is_in_hospital:
@@ -827,6 +882,13 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		if isinstance(data, gmEMRStructItems.cHospitalStay):
 			return data.format()
 
+		if isinstance(data, gmExternalCare.cExternalCareItem):
+			return u'\n'.join(data.format (
+				with_health_issue = True,
+				with_address = True,
+				with_comms = True
+			))
+
 		if isinstance(data, gmDemographicRecord.cPatientAddress):
 			return u'\n'.join(data.format())
 
@@ -890,7 +952,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				if isinstance(data, gmStaff.cStaff):
 					pass
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmNotebookedPatientEditionPlugin')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmNotebookedPatientEditionPlugin')
 	#-----------------------------------------------------
 	#-----------------------------------------------------
 	def __refresh_problems(self, patient=None):
@@ -902,6 +964,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 		]
 
 		list_items = []
+		list_data = []
 		for problem in problems:
 			if problem['type'] == 'issue':
 				issue = emr.problem2issue(problem)
@@ -911,7 +974,6 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				else:
 					last = last_encounter['last_affirmed'].strftime('%m/%Y')
 				list_items.append(u'%s: %s' % (problem['problem'], last))
-
 			elif problem['type'] == 'episode':
 				epi = emr.problem2episode(problem)
 				last_encounter = emr.get_last_encounter(episode_id = epi['pk_episode'])
@@ -920,11 +982,27 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 				else:
 					last = last_encounter['last_affirmed'].strftime('%m/%Y')
 				list_items.append(u'%s: %s' % (problem['problem'], last))
+			list_data.append(problem)
+
+		care = emr.get_external_care_items()
+		for item in care:
+			if item['pk_health_issue'] is not None:
+				continue
+			list_items.append(_('extrnl: %s (%s@%s)') % (
+				item['issue'],
+				item['unit'],
+				item['organization']
+			))
+			list_data.append(item)
 
 		self._LCTRL_problems.set_string_items(items = list_items)
-		self._LCTRL_problems.set_data(data = problems)
+		self._LCTRL_problems.set_data(data = list_data)
 	#-----------------------------------------------------
 	def _calc_problem_list_item_tooltip(self, data):
+
+		if isinstance(data, gmExternalCare.cExternalCareItem):
+			return u'\n'.join(data.format(with_health_issue = True, with_comms = True))
+
 		emr = gmPerson.gmCurrentPatient().get_emr()
 
 		if data['type'] == 'issue':
@@ -971,7 +1049,7 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 					gmEMRStructWidgets.edit_episode(parent = self, episode = emr.problem2episode(data))
 					return
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmEMRBrowserPlugin')
+		gmDispatcher.send(signal = 'display_widget', name = 'gmEMRBrowserPlugin')
 	#-----------------------------------------------------
 	#-----------------------------------------------------
 	def __refresh_identity(self, patient=None):
@@ -1025,30 +1103,34 @@ class cPatientOverviewPnl(wxgPatientOverviewPnl.wxgPatientOverviewPnl, gmRegetMi
 	#-----------------------------------------------------
 	def _on_identity_item_activated(self, event):
 		data = self._LCTRL_identity.get_selected_item_data(only_one = True)
-		if data is not None:
-			# <ctrl> down ?
-			if wx.GetKeyState(wx.WXK_CONTROL):
-				if isinstance(data, gmPerson.cPersonName):
-					ea = gmDemographicsWidgets.cPersonNameEAPnl(self, -1, name = data)
-					dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
-					dlg.SetTitle(_('Cloning name'))
-					dlg.ShowModal()
-					return
-				if isinstance(data, type({})):
-					key = data.keys()[0]
-					val = data[key]
-					if key == 'id':
-						ea = gmDemographicsWidgets.cExternalIDEditAreaPnl(self, -1, external_id = val)
-						ea.identity = gmPerson.gmCurrentPatient()
-						dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
-						dlg.SetTitle(_('Editing external ID'))
-						dlg.ShowModal()
-						return
-					if key == 'job':
-						gmDemographicsWidgets.edit_occupation()
-						return
+		if data is None:
+			gmDispatcher.send(signal = 'display_widget', name = 'gmNotebookedPatientEditionPlugin')
 
-		wx.CallAfter(gmDispatcher.send, signal = 'display_widget', name = 'gmNotebookedPatientEditionPlugin')
+		# <ctrl> down ?
+		if not wx.GetKeyState(wx.WXK_CONTROL):
+			gmDispatcher.send(signal = 'display_widget', name = 'gmNotebookedPatientEditionPlugin')
+
+		# <ctrl> down !
+		if isinstance(data, gmPerson.cPersonName):
+			ea = gmDemographicsWidgets.cPersonNameEAPnl(self, -1, name = data)
+			dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
+			dlg.SetTitle(_('Cloning name'))
+			dlg.ShowModal()
+			return
+
+		if isinstance(data, type({})):
+			key = data.keys()[0]
+			val = data[key]
+			if key == 'id':
+				ea = gmDemographicsWidgets.cExternalIDEditAreaPnl(self, -1, external_id = val)
+				ea.id_holder = gmPerson.gmCurrentPatient()
+				dlg = gmEditArea.cGenericEditAreaDlg2(self, -1, edit_area = ea, single_entry = True)
+				dlg.SetTitle(_('Editing external ID'))
+				dlg.ShowModal()
+				return
+			if key == 'job':
+				gmDemographicsWidgets.edit_occupation()
+				return
 #============================================================
 # main
 #------------------------------------------------------------
