@@ -29,6 +29,7 @@ from Gnumed.business import gmDemographicRecord
 
 from Gnumed.wxpython import gmEditArea
 from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmEncounterWidgets
 from Gnumed.wxpython.gmDemographicsWidgets import _validate_dob_field, _validate_tob_field, _empty_dob_allowed
 
 
@@ -37,6 +38,18 @@ _log = logging.getLogger('gm.patient')
 #============================================================
 def create_new_person(parent=None, activate=False):
 
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	if activate:			# meaning we will switch away from the current patient if any
+		msg = _(
+			u'Before creating a new person review the encounter details\n'
+			u'of the patient you just worked on:\n'
+		)
+		gmEncounterWidgets.sanity_check_encounter_of_active_patient(parent = parent, msg = msg)
+
+		msg = _('Edit the current encounter of the patient you are ABOUT TO LEAVE:')
+
 	dbcfg = gmCfg.cCfgSQL()
 
 	def_region = dbcfg.get2 (
@@ -57,9 +70,6 @@ def create_new_person(parent=None, activate=False):
 		if len(countries) == 1:
 			def_country = countries[0]['code_country']
 
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-
 	ea = cNewPatientEAPnl(parent = parent, id = -1, country = def_country, region = def_region)
 	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea, single_entry = True)
 	dlg.SetTitle(_('Adding new person'))
@@ -491,8 +501,8 @@ class cNewPatientEAPnl(wxgNewPatientEAPnl.wxgNewPatientEAPnl, gmEditArea.cGeneri
 					street = self._PRW_street.GetValue().strip(),
 					postcode = self._PRW_zip.GetValue().strip(),
 					urb = self._PRW_urb.GetValue().strip(),
-					state = self._PRW_region.GetData(),
-					country = self._PRW_country.GetData(),
+					region_code = self._PRW_region.GetData(),
+					country_code = self._PRW_country.GetData(),
 					subunit = gmTools.none_if(self._TCTRL_unit.GetValue().strip(), u''),
 					id_type = self._PRW_type.GetData()
 				)
