@@ -90,29 +90,24 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 		return
 	#----------------------------------------------
 	def _on_tag_change(self):
-		wx.CallAfter(self.__update_tags)
+		self.__update_tags()
 	#----------------------------------------------
 	def _on_name_identity_change(self):
-		wx.CallAfter(self.__update_age_label)
+		self.__update_age_label()
 	#----------------------------------------------
 	def _on_post_patient_selection(self, **kwargs):
-		# needed because GUI stuff can't be called from a thread (and that's
-		# where we are coming from via backend listener -> dispatcher)
-		wx.CallAfter(self.__on_post_patient_selection, **kwargs)
+		self.__update_age_label()
+		self.__update_allergies()
+		self.__update_tags()
 	#-------------------------------------------------------
 	def _on_allergies_change(self, **kwargs):
-		wx.CallAfter(self.__update_allergies)
+		self.__update_allergies()
 	#-------------------------------------------------------
 	def _on_focus_patient_search(self, **kwargs):
 		wx.CallAfter(self._TCTRL_patient_selector.SetFocus)
 	#-------------------------------------------------------
 	# internal API
 	#-------------------------------------------------------
-	def __on_post_patient_selection(self, **kwargs):
-		self.__update_age_label()
-		self.__update_allergies()
-		self.__update_tags()
-	#-------------------------------------------------------
 	def __update_tags(self):
 		self._PNL_tags.refresh(patient = self.curr_pat)
 	#-------------------------------------------------------
@@ -165,17 +160,17 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 		# patient birthday ?
 		if self.curr_pat.get_formatted_dob(format = '%m-%d') == now.strftime('%m-%d'):
 			template = _('%(sex)s  %(dob)s (%(age)s today !)')
-			tt += _("\nToday is the patient's birtday !\n\n")
+			tt += _("\nToday is the patient's birthday !\n\n")
 		else:
 			if self.curr_pat.current_birthday_passed():
 				template = u'%(sex)s  %(dob)s%(l_arr)s (%(age)s)'
 				tt += _(u'Birthday: %s ago\n') % gmDateTime.format_apparent_age_medically (
-					age = gmDateTime.calculate_apparent_age(start = self.curr_pat.dob_this_year, end = now)
+					age = gmDateTime.calculate_apparent_age(start = self.curr_pat.birthday_this_year, end = now)
 				)
 			else:
 				template = u'%(sex)s  %(r_arr)s%(dob)s (%(age)s)'
-				tt += _(u'Birtday: in %s\n') % gmDateTime.format_apparent_age_medically (
-					age = gmDateTime.calculate_apparent_age(start = now, end = self.curr_pat.dob_this_year)
+				tt += _(u'Birthday: in %s\n') % gmDateTime.format_apparent_age_medically (
+					age = gmDateTime.calculate_apparent_age(start = now, end = self.curr_pat.birthday_this_year)
 				)
 
 		tt += _('Age: %s\n') % self.curr_pat['medical_age']
@@ -262,7 +257,6 @@ class cTopPnl(wxgTopPnl.wxgTopPnl):
 
 #===========================================================	
 if __name__ == "__main__":
-	wx.InitAllImageHandlers()
 	app = wxPyWidgetTester(size = (400, 200))
 	app.SetWidget(cMainTopPanel, -1)
 	app.SetWidget(cTopPanel, -1)
